@@ -34,6 +34,7 @@ namespace Gwenview {
 
 class DocumentPanel;
 class ContextManager;
+class MessageBubble;
 
 class MainWindow : public KXmlGuiWindow {
 Q_OBJECT
@@ -55,9 +56,16 @@ public:
 
 	void setDistractionFreeMode(bool);
 
+	void showMessageBubble(MessageBubble*);
+
 public Q_SLOTS:
 	void showStartPage();
 
+	/**
+	 * Go to url, without changing current mode
+	 */
+	void goToUrl(const KUrl&);
+
 Q_SIGNALS:
 	void viewModeChanged();
 
@@ -72,6 +80,8 @@ protected:
 	virtual QSize sizeHint() const;
 	virtual void showEvent(QShowEvent*);
 	virtual void resizeEvent(QResizeEvent*);
+	virtual void saveProperties(KConfigGroup&);
+	virtual void readProperties(const KConfigGroup&);
 
 private Q_SLOTS:
 	void setActiveViewModeAction(QAction* action);
@@ -81,7 +91,6 @@ private Q_SLOTS:
 	void slotStartPageUrlSelected(const KUrl&);
 
 	void openDocumentUrl(const KUrl&);
-	void goToUrl(const KUrl&);
 	void goUp();
 	void toggleSideBar(bool visible);
 	void updateToggleSideBarAction();
@@ -108,6 +117,12 @@ private Q_SLOTS:
 
 	void slotSelectionChanged();
 
+	/**
+	 * Check if a selected index is among the changed indexes, if so, update
+	 * context
+	 */
+	void slotDataChanged(const QModelIndex&, const QModelIndex&);
+
 	void goToPrevious();
 	void goToNext();
 	void updatePreviousNextActions();
@@ -124,7 +139,6 @@ private Q_SLOTS:
 
 	void showDocumentInFullScreen(const KUrl&);
 
-	void generateThumbnailForUrl(const KUrl&);
 	void showConfigDialog();
 	void loadConfig();
 	void print();
