@@ -22,6 +22,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // Qt
 #include <QApplication>
 #include <QImage>
+#include <QQueue>
 #include <QUndoStack>
 
 // KDE
@@ -30,9 +31,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <kurl.h>
 
 // Local
+#include "documentjob.h"
 #include "emptydocumentimpl.h"
 #include "imagemetainfomodel.h"
 #include "loadingdocumentimpl.h"
+#include "loadingjob.h"
+#include "savejob.h"
 
 namespace Gwenview {
 
@@ -40,6 +44,7 @@ struct DocumentPrivate {
 	AbstractDocumentImpl* mImpl;
 	KUrl mUrl;
 	bool mKeepRawData;
+	QQueue<DocumentJob*> mJobQueue;
 
 	/**
 	 * @defgroup imagedata should be reset in reload()
@@ -75,12 +80,9 @@ Document::Document(const KUrl& url)
 	d->mImpl = 0;
 	d->mUrl = url;
 	d->mKeepRawData = false;
-	d->mKind = MimeTypeUtils::KIND_UNKNOWN;
-
 	connect(&d->mUndoStack, SIGNAL(indexChanged(int)), SLOT(slotUndoIndexChanged()) );
-	KFileItem fileItem(KFileItem::Unknown, KFileItem::Unknown, url);
-	d->mImageMetaInfoModel.setFileItem(fileItem);
-	switchToImpl(new LoadingDocumentImpl(this));
+
+	reload();
 }
 
 
@@ -209,7 +211,7 @@ void Document::setKeepRawData(bool value) {
 
 
 void Document::waitUntilLoaded() {
-	loadFullImage();
+	startLoadingFullImage();
 	while (true) {
 		LoadingState state = loadingState();
 		if (state == Loaded || state == LoadingFailed) {
@@ -220,20 +222,33 @@ void Document::waitUntilLoaded() {
 }
 
 
-bool Document::save(const KUrl& url, const QByteArray& format) {
+DocumentJob* Document::save(const KUrl& url, const QByteArray& format) {
 	waitUntilLoaded();
-	bool ok = d->mImpl->save(url, format);
-	if (ok) {
-		// Use QueuedConnection to ensure we are (more or less :/) thread-safe.
-		// FIXME: This method should be turned into an asynchronous call to
-		// reduce thread-safe potential issues.
-		QMetaObject::invokeMethod(this, "saved", Qt::QueuedConnection, Q_ARG(KUrl, d->mUrl));
-		QMetaObject::invokeMethod(&d->mUndoStack, "setClean", Qt::QueuedConnection);
+	DocumentJob* job = d->mImpl->save(url, format);
+	job->setProperty("oldUrl", d->mUrl);
+	job->setProperty("newUrl", url);
+	if (!job) {
+		kWarning() << "Implementation does not support saving!";
+		return false;
 	}
-
-	return ok;
+	connect(job, SIGNAL(result(KJob*)), SLOT(slotSaveResult(KJob*)));
+	enqueueJob(job);
+	return job;
 }
 
+
+void Document::slotSaveResult(KJob* job) {
+	if (job->error()) {
+		setErrorString(job->errorString());
+	} else {
+		d->mUndoStack.setClean();
+		SaveJob* saveJob = static_cast<SaveJob*>(job);
+		d->mUrl = saveJob->newUrl();
+		saved(saveJob->oldUrl(), d->mUrl);
+	}
+}
+
+
 QByteArray Document::format() const {
 	return d->mFormat;
 }
@@ -327,10 +342,11 @@ ImageMetaInfoModel* Document::metaInfo() const {
 }
 
 
-void Document::loadFullImage() {
+void Document::startLoadingFullImage() {
 	LoadingState state = loadingState();
 	if (state <= MetaInfoLoaded) {
 		// Schedule full image loading
+		enqueueJob(new LoadingJob);
 		d->scheduleImageLoading(1);
 	} else if (state == Loaded) {
 		return;
@@ -394,7 +410,7 @@ void Document::slotUndoIndexChanged() {
 		// If user just undid all his changes this does not really correspond
 		// to a save, but it's similar enough as far as Document users are
 		// concerned
-		saved(d->mUrl);
+		saved(d->mUrl, d->mUrl);
 	} else {
 		modified(d->mUrl);
 	}
@@ -420,5 +436,36 @@ void Document::stopAnimation() {
 	return d->mImpl->stopAnimation();
 }
 
+void Document::enqueueJob(DocumentJob* task) {
+	d->mJobQueue.enqueue(task);
+	task->setDocument(Ptr(this));
+	connect(task, SIGNAL(destroyed(QObject*)),
+		SLOT(slotJobDestroyed(QObject*)));
+	if (d->mJobQueue.size() == 1) {
+		task->start();
+		busyChanged(d->mUrl, true);
+	}
+}
+
+void Document::slotJobDestroyed(QObject* job) {
+	Q_ASSERT(!d->mJobQueue.isEmpty());
+	if (d->mJobQueue.head() != job) {
+		// Job was killed before it even got started, just remove it from the
+		// queue and move along
+		d->mJobQueue.removeAll(static_cast<DocumentJob*>(job));
+		return;
+	}
+	d->mJobQueue.dequeue();
+	if (d->mJobQueue.isEmpty()) {
+		busyChanged(d->mUrl, false);
+		allTasksDone();
+	} else {
+		d->mJobQueue.head()->start();
+	}
+}
+
+bool Document::isBusy() const {
+	return !d->mJobQueue.isEmpty();
+}
 
 } // namespace
