@@ -62,11 +62,12 @@ protected:
 
 Q_SIGNALS:
 	void done( const QImage&, const QSize&);
+	void thumbnailReadyToBeCached(const QString& thumbnailPath, const QImage&);
 
 private:
 	bool testCancel();
-	void loadThumbnail();
-	void storeThumbnailInCache();
+	bool loadThumbnail(bool* needCaching);
+	void cacheThumbnail();
 	QImage mImage;
 	QString mPixPath;
 	QString mThumbnailPath;
@@ -82,6 +83,27 @@ private:
 	bool mCancel;
 };
 
+/**
+ * Store thumbnails to disk when done generating them
+ */
+class ThumbnailCache : public QThread {
+Q_OBJECT
+public:
+	// Return thumbnail if it has still not been stored
+	QImage value(const QString&) const;
+
+public Q_SLOTS:
+	void queueThumbnail(const QString&, const QImage&);
+
+protected:
+	void run();
+
+private:
+	typedef QHash<QString, QImage> Cache;
+	Cache mCache;
+	mutable QMutex mMutex;
+};
+
 /**
  * A job that determines the thumbnails for the images in the current directory
  */
@@ -140,6 +162,11 @@ public:
 	 */
 	static void deleteImageThumbnail(const KUrl& url);
 
+	/**
+	 * Move a thumbnail to match a file move
+	 */
+	static void moveThumbnail(const KUrl& oldUrl, const KUrl& newUrl);
+
 Q_SIGNALS:
 	/**
 	 * Emitted when the thumbnail for the @p item has been loaded
@@ -187,6 +214,8 @@ private:
 	void startCreatingThumbnail(const QString& path);
 
 	void emitThumbnailLoaded(const QImage& img, const QSize& size);
+
+	QImage loadThumbnailFromCache() const;
 };
 
 } // namespace
