@@ -27,7 +27,7 @@
 class DisplayNumber : public QLabel  {
    Q_OBJECT
 public: 
-  DisplayNumber(QWidget *parent=0, int digits=0, int prec=0, const char *name=0);
+  DisplayNumber(QWidget *parent=0, int digits=0, int prec=0);
   ~DisplayNumber();
   void setFont( const QFont & f );
   void setNum(double num);
