@@ -1,25 +1,26 @@
 /* -*- C++ -*-
  * File: 4channels.cpp
- * Copyright 2008-2009 LibRaw LLC (info@libraw.org)
+ * Copyright 2008-2010 LibRaw LLC (info@libraw.org)
  * Created: Mon Feb 09, 2009
  *
  * LibRaw sample
  * Generates 4 TIFF file from RAW data, one file per channel
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+
+LibRaw is free software; you can redistribute it and/or modify
+it under the terms of the one of three licenses as you choose:
+
+1. GNU LESSER GENERAL PUBLIC LICENSE version 2.1
+   (See file LICENSE.LGPL provided in LibRaw distribution archive for details).
+
+2. COMMON DEVELOPMENT AND DISTRIBUTION LICENSE (CDDL) Version 1.0
+   (See file LICENSE.CDDL provided in LibRaw distribution archive for details).
+
+3. LibRaw Software License 27032010
+   (See file LICENSE.LibRaw.pdf provided in LibRaw distribution archive for details).
+
+
+
  */
 #include <stdio.h>
 #include <string.h>
@@ -90,7 +91,7 @@ int main(int ac, char *av[])
                         autoscale=1;
                     else if(av[i][1]=='B' && av[i][2]==0)
                         {
-                            filtering_mode |= (LIBRAW_FILTERING_NOZEROES | LIBRAW_FILTERING_NOBLACKS);
+                            filtering_mode |= (LIBRAW_FILTERING_NOZEROES);
                             black_subtraction=0;
                         }
                     else if(av[i][1]=='N' && av[i][2]==0)
@@ -111,24 +112,14 @@ int main(int ac, char *av[])
                     fprintf(stderr,"Cannot open %s: %s\n",av[i],libraw_strerror(ret));
                     continue; // no recycle b/c open file will recycle itself
                 }
-            if(P1.is_foveon)
-                {
-                    printf("Cannot process foveon image %s\n",av[i]);
-                    continue ;
-                }
             if( (ret = RawProcessor.unpack() ) != LIBRAW_SUCCESS)
                 {
                     fprintf(stderr,"Cannot unpack %s: %s\n",av[i],libraw_strerror(ret));
                     continue;
                 }
-            if(black_subtraction && C.black>0)
+            if(black_subtraction)
                 {
-                    for(int j=0; j<S.iheight*S.iwidth; j++)
-                        for(int c = 0; c< 4; c++)
-                            if(RawProcessor.imgdata.image[j][c]>C.black)
-                                RawProcessor.imgdata.image[j][c]-=C.black;
-                            else
-                                RawProcessor.imgdata.image[j][c]=0;
+                    RawProcessor.subtract_black();
                 }
 
             if(autoscale)
