@@ -74,8 +74,6 @@ namespace KSaneIface
         // Start the scanning with sane_start
         status = sane_start(m_saneHandle);
         
-        m_saneStartDone = true;
-        
         if (status != SANE_STATUS_GOOD) {
             kDebug() << "sane_start=" << sane_strstatus(status);
             sane_cancel(m_saneHandle);
@@ -125,6 +123,10 @@ namespace KSaneIface
         m_frameRead   = 0;
         m_px_c_index  = 0;
         m_frame_t_count = 0;
+
+        // set the m_saneStartDone here so the new QImage gets allocated before updating the preview.
+        m_saneStartDone = true;
+        
         
         while (m_readStatus == READ_ON_GOING) {
             readData();
@@ -186,7 +188,7 @@ namespace KSaneIface
                         sane_cancel(m_saneHandle);
                         return;
                     }
-                    kDebug() << "New Frame";
+                    //kDebug() << "New Frame";
                     m_frameRead = 0;
                     m_pixel_x     = 0;
                     m_pixel_y     = 0;
@@ -242,7 +244,7 @@ namespace KSaneIface
                             // resize the image
                             *m_img = m_img->copy(0, 0, m_img->width(), m_img->height() + m_img->width());
                         }
-                        for (j=7; j>=0; j--) {
+                        for (j=7; j>=0; --j) {
                             if ((m_readData[i] & (1<<j)) == 0) {
                                 m_img->setPixel(m_pixel_x,
                                                 m_pixel_y,
@@ -372,7 +374,7 @@ namespace KSaneIface
                         return;
                     }
                     break;
-                    
+
                 case SANE_FRAME_GREEN:
                     if (m_params.depth == 8) {
                         for (int i=0; i<read_bytes; i++) {
@@ -401,7 +403,7 @@ namespace KSaneIface
                         return;
                     }
                     break;
-                    
+
                 case SANE_FRAME_BLUE:
                     if (m_params.depth == 8) {
                         for (int i=0; i<read_bytes; i++) {
@@ -431,19 +433,17 @@ namespace KSaneIface
                     }
                     break;
         }
-        
+
         kWarning() << "Format" << m_params.format
         << "and depth" << m_params.format
         << "is not yet suppoeted by libksane!";
         m_readStatus = READ_ERROR;
         return;
     }
-    
-    
+
     bool KSanePreviewThread::saneStartDone()
     {
         return   m_saneStartDone;
     }
-    
+
 }  // NameSpace KSaneIface
-    
\ No newline at end of file
