@@ -102,6 +102,7 @@ void KSaneWidgetPrivate::clearDeviceOptions()
     m_optGamG       = 0;
     m_optGamB       = 0;
     m_optPreview    = 0;
+    m_optWaitForBtn = 0;
     m_scanOngoing   = false;
     m_closeDevicePending = false;
     
@@ -261,12 +262,17 @@ void KSaneWidgetPrivate::createOptInterface()
             basic_layout->addWidget(option->widget());
         }
     }
-    
+
     // save a pointer to the preview option if possible
     if ((option = getOption(SANE_NAME_PREVIEW)) != 0) {
         m_optPreview = option;
     }
 
+    // save a pointer to the "wait-for-button" option if possible (Note: No translation)
+    if ((option = getOption("wait-for-button")) != 0) {
+        m_optWaitForBtn = option;
+    }
+
     // scan area (Do not add the widgets)
     if ((option = getOption(SANE_NAME_SCAN_TL_X)) != 0) {
         m_optTlX = option;
@@ -432,11 +438,11 @@ void KSaneWidgetPrivate::optReload()
         m_optList.at(i)->readValue();
     }
     // estimate the preview size and create an empty image
-    // this is done so that you can select scanarea without
+    // this is done so that you can select scan area without
     // having to scan a preview.
     updatePreviewSize();
     
-    // encsure that we do not get a scrollbar at the bottom of the option of the options
+    // ensure that we do not get a scrollbar at the bottom of the option of the options
     int min_width = m_basicOptsTab->sizeHint().width();
     if (min_width < m_otherOptsTab->sizeHint().width()) {
         min_width = m_otherOptsTab->sizeHint().width();
@@ -444,6 +450,7 @@ void KSaneWidgetPrivate::optReload()
     
     m_optsTabWidget->setMinimumWidth(min_width + 32);
     
+    m_previewViewer->zoom2Fit();
 }
 
 void KSaneWidgetPrivate::valReload()
@@ -562,13 +569,16 @@ void KSaneWidgetPrivate::updatePreviewSize()
         m_optBrY->getMaxValue(max_y);
     }
     if ((max_x == m_previewWidth) && (max_y == m_previewHeight)) {
-        //kDebug() << "no preview width";
+        //kDebug() << "no preview size change";
         return;
     }
     
+    // The preview size has changed 
     m_previewWidth  = max_x;
     m_previewHeight = max_y;
+
     // set the scan area to the whole area
+    m_previewViewer->clearSelections();
     if (m_optTlX != 0) {
         m_optTlX->setValue(0);
     }
@@ -614,10 +624,6 @@ void KSaneWidgetPrivate::startPreviewScan()
     if (m_optDepth != 0) m_optDepth->storeCurrentData();
     if (m_optRes != 0) m_optRes->storeCurrentData();
     if (m_optResY != 0) m_optResY->storeCurrentData();
-    if (m_optTlX != 0) m_optTlX->storeCurrentData();
-    if (m_optTlY != 0) m_optTlY->storeCurrentData();
-    if (m_optBrX != 0) m_optBrX->storeCurrentData();
-    if (m_optBrY != 0) m_optBrY->storeCurrentData();
     if (m_optPreview != 0) m_optPreview->storeCurrentData();
     
     // check if we can modify the selection
@@ -688,7 +694,14 @@ void KSaneWidgetPrivate::startPreviewScan()
         valReload();
     }
     
+    // clear the preview
+    m_previewViewer->clearHighlight();
+    m_previewViewer->clearSelections();
+    m_previewImg.fill(0xFFFFFFFF);
+    updatePreviewSize();
+    
     setBusy(true);
+    
     m_progressBar->setValue(0);
     m_isPreview = true;
     m_previewThread->setPreviewInverted(m_invertColors->isChecked());
@@ -713,10 +726,6 @@ void KSaneWidgetPrivate::previewScanDone()
     if (m_optDepth != 0) m_optDepth->restoreSavedData();
     if (m_optRes != 0) m_optRes->restoreSavedData();
     if (m_optResY != 0) m_optResY->restoreSavedData();
-    if (m_optTlX != 0) m_optTlX->restoreSavedData();
-    if (m_optTlY != 0) m_optTlY->restoreSavedData();
-    if (m_optBrX != 0) m_optBrX->restoreSavedData();
-    if (m_optBrY != 0) m_optBrY->restoreSavedData();
     if (m_optPreview != 0) m_optPreview->restoreSavedData();
     
     if (m_autoSelect) {
@@ -727,13 +736,7 @@ void KSaneWidgetPrivate::previewScanDone()
     m_scanOngoing = false;
     m_updProgressTmr.stop();
     
-    if (m_progressBar->value() == 0) {
-        m_previewViewer->setQImage(&m_previewImg);
-        m_previewViewer->zoom2Fit();
-    }
-    else {
-        m_previewViewer->updateImage();
-    }
+    m_previewViewer->updateImage();
     
     emit scanDone(KSaneWidget::NoError, "");
     
@@ -758,6 +761,7 @@ void KSaneWidgetPrivate::startFinalScan()
 
         // reead the selection from the viewer
         m_previewViewer->selectionAt(m_selIndex, x1,y1,x2,y2);
+        m_previewViewer->setHighlightArea(x1,y1,x2,y2);
         m_selIndex++;
  
         // calculate the option values
@@ -811,16 +815,33 @@ void KSaneWidgetPrivate::oneFinalScanDone()
                          (int)getImgFormat(params));
 
         // now check if we should have automatic ADF batch scaning
-        QString source;
         if (m_optSource){
+            QString source;
             m_optSource->getValue(source);
+
+            if (source == "Automatic Document Feeder") {
+                // in batch mode only one area can be scanned per page
+                //kDebug() << "source == \"Automatic Document Feeder\"";
+                m_updProgressTmr.start();
+                m_scanThread->start();
+                return;
+            }
         }
-        if (source == "Automatic Document Feeder") {
-            // in batch mode only one area can be scanned per page
-            //kDebug() << "source == \"Automatic Document Feeder\"";
-            m_updProgressTmr.start();
-            m_scanThread->start();
-            return;
+
+        // Check if we have a "wait for button" batch scanning
+        if (m_optWaitForBtn) {
+            kDebug() << m_optWaitForBtn->name();
+            QString wait;
+            m_optWaitForBtn->getValue(wait);
+
+            kDebug() << "wait ==" << wait;
+            if (wait == "true") {
+                // in batch mode only one area can be scanned per page
+                //kDebug() << "source == \"Automatic Document Feeder\"";
+                m_updProgressTmr.start();
+                m_scanThread->start();
+                return;
+            }
         }
 
         // not batch scan, call sane_cancel to be able to change parameters.
@@ -837,8 +858,11 @@ void KSaneWidgetPrivate::oneFinalScanDone()
                 m_optBrX->getMaxValue(max_x);
                 m_optBrY->getMaxValue(max_y);
                 
-                // reead the selection from the viewer
+                // read the selection from the viewer
                 m_previewViewer->selectionAt(m_selIndex, x1,y1,x2,y2);
+
+                // set the highlight
+                m_previewViewer->setHighlightArea(x1,y1,x2,y2);
                 
                 // calculate the option values
                 x1 *= max_x; y1 *= max_y;
@@ -890,6 +914,8 @@ void KSaneWidgetPrivate::oneFinalScanDone()
     
     sane_cancel(m_saneHandle);
 
+    // clear the highlight
+    m_previewViewer->setHighlightArea(0,0,1,1);
     setBusy(false);
     m_scanOngoing = false;
     
@@ -948,15 +974,17 @@ void KSaneWidgetPrivate::updateProgress()
     int progress;
     if (m_isPreview) {
         progress = m_previewThread->scanProgress();
-        if (!m_progressBar->isVisible() && (m_previewThread->saneStartDone())) {
-            m_warmingUp->hide();
-            m_activityFrame->show();
-            m_previewViewer->setQImage(&m_previewImg);
-            m_previewViewer->zoom2Fit();
-            
-        }
-        else {
-            m_previewViewer->updateImage();
+        if (m_previewThread->saneStartDone()) {
+            if (!m_progressBar->isVisible()) {
+                m_warmingUp->hide();
+                m_activityFrame->show();
+                // the image size might have changed
+                m_previewViewer->setQImage(&m_previewImg);
+                m_previewViewer->zoom2Fit();
+            }
+            else {
+                m_previewViewer->updateImage();
+            }
         }
     }
     else {
@@ -965,6 +993,7 @@ void KSaneWidgetPrivate::updateProgress()
             m_activityFrame->show();
         }
         progress = m_scanThread->scanProgress();
+        m_previewViewer->setHighlightShown(progress);
     }
     
     m_progressBar->setValue(progress);
