@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007 by John Layt <john@layt.net>                       *
+ *   Copyright (C) 2007, 2010 by John Layt <john@layt.net>                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,6 +18,7 @@
 #include <QtCore/QString>
 
 #include <okular/core/okular_export.h>
+#include <okular/core/generator.h>
 
 class QPrinter;
 class QSize;
@@ -63,6 +64,7 @@ public:
      *          -8 if empty file name
      *          -7 if unable to find file
      *          -6 if invalid printer state
+     *          -5 if print to file copy failed
      *          -2 if the KProcess could not be started
      *          -1 if the KProcess crashed
      *          otherwise the KProcess exit code
@@ -87,6 +89,7 @@ public:
      *          -8 if empty file list
      *          -7 if unable to find a file
      *          -6 if invalid printer state
+     *          -5 if print to file copy failed
      *          -2 if the KProcess could not be started
      *          -1 if the KProcess crashed
      *          otherwise the KProcess exit code
@@ -95,6 +98,17 @@ public:
     static int printFiles( QPrinter &printer, const QStringList &fileList,
                            FileDeletePolicy fileDeletePolicy = FilePrinter::ApplicationDeletesFiles );
 
+    /** Return the list of pages selected by the user in the Print Dialog
+     *
+     * @param printer the print settings to use
+     * @param lastPage the last page number, needed if AllPages option is selected
+     * @param currentPage the current page number, needed if CurrentPage option is selected
+     * @param selectedPageList list of pages to use if Selection option is selected
+     * @returns Returns list of pages to print
+     */
+    static QList<int> pageList( QPrinter &printer, int lastPage,
+                                int currentPage, const QList<int> &selectedPageList );
+
     /** Return the list of pages selected by the user in the Print Dialog
      *
      * @param printer the print settings to use
@@ -144,6 +158,12 @@ public:
      */
     static QSize psPaperSize( QPrinter &printer );
 
+    /**
+     * Convert the code returned by printFile* to PrintError
+     * @since 0.11 (KDE 4.5)
+     */
+    static Generator::PrintError printError( int c );
+
 protected:
 
     bool detectCupsService();
