@@ -114,7 +114,11 @@ Reviews::Reviews( QWidget * parent, Okular::Document * document )
 
     m_view->setModel( m_authorProxy );
 
-    vLayout->addWidget( new KTreeViewSearchLine( this, m_view ) );
+    m_searchLine = new KTreeViewSearchLine( this, m_view );
+    m_searchLine->setCaseSensitivity( Okular::Settings::self()->reviewsSearchCaseSensitive() ? Qt::CaseSensitive : Qt::CaseInsensitive );
+    m_searchLine->setRegularExpression( Okular::Settings::self()->reviewsSearchRegularExpression() );
+    connect( m_searchLine, SIGNAL( searchOptionsChanged() ), this, SLOT( saveSearchOptions() ) );
+    vLayout->addWidget( m_searchLine );
     vLayout->addWidget( m_view );
     vLayout->addWidget( toolBar );
 
@@ -160,6 +164,13 @@ void Reviews::notifyViewportChanged( bool )
 }
 //END DocumentObserver Notifies 
 
+void Reviews::reparseConfig()
+{
+    m_searchLine->setCaseSensitivity( Okular::Settings::reviewsSearchCaseSensitive() ? Qt::CaseSensitive : Qt::CaseInsensitive );
+    m_searchLine->setRegularExpression( Okular::Settings::reviewsSearchRegularExpression() );
+    m_view->update();
+}
+
 //BEGIN GUI Slots -> requestListViewUpdate
 void Reviews::slotPageEnabled( bool on )
 {
@@ -217,6 +228,28 @@ void Reviews::activated( const QModelIndex &index )
     m_document->setViewport( vp, -1, true );
 }
 
+QModelIndexList Reviews::retrieveAnnotations(const QModelIndex& idx) const
+{
+    QModelIndexList ret;
+    if ( idx.isValid() )
+    {
+        if ( idx.model()->hasChildren( idx ) )
+        {
+            int rowCount = idx.model()->rowCount( idx );
+            for ( int i = 0; i < rowCount; i++ )
+            {
+                ret += retrieveAnnotations( idx.child( i, idx.column() ) );
+            }
+        }
+        else
+        {
+            ret += idx;
+        }
+    }
+    
+    return ret;
+}
+
 void Reviews::contextMenuRequested( const QPoint &pos )
 {
     AnnotationPopup popup( m_document, this );
@@ -228,17 +261,29 @@ void Reviews::contextMenuRequested( const QPoint &pos )
     QModelIndexList indexes = m_view->selectionModel()->selectedIndexes();
     Q_FOREACH ( const QModelIndex &index, indexes )
     {
-        const QModelIndex authorIndex = m_authorProxy->mapToSource( index );
-        const QModelIndex filterIndex = m_groupProxy->mapToSource( authorIndex );
-        const QModelIndex annotIndex = m_filterProxy->mapToSource( filterIndex );
-        Okular::Annotation *annotation = m_model->annotationForIndex( annotIndex );
-        if ( annotation ) {
-            const int pageNumber = m_model->data( annotIndex, AnnotationModel::PageRole ).toInt();
-            popup.addAnnotation( annotation, pageNumber );
+        QModelIndexList annotations = retrieveAnnotations(index);
+        Q_FOREACH ( const QModelIndex &idx, annotations )
+        {
+            const QModelIndex authorIndex = m_authorProxy->mapToSource( idx );
+            const QModelIndex filterIndex = m_groupProxy->mapToSource( authorIndex );
+            const QModelIndex annotIndex = m_filterProxy->mapToSource( filterIndex );
+            Okular::Annotation *annotation = m_model->annotationForIndex( annotIndex );
+            if ( annotation )
+            {
+                const int pageNumber = m_model->data( annotIndex, AnnotationModel::PageRole ).toInt();
+                popup.addAnnotation( annotation, pageNumber );
+            }
         }
     }
 
     popup.exec( m_view->viewport()->mapToGlobal( pos ) );
 }
 
+void Reviews::saveSearchOptions()
+{
+    Okular::Settings::setReviewsSearchRegularExpression( m_searchLine->regularExpression() );
+    Okular::Settings::setReviewsSearchCaseSensitive( m_searchLine->caseSensitivity() == Qt::CaseSensitive ? true : false );
+    Okular::Settings::self()->writeConfig();
+}
+
 #include "side_reviews.moc"
